"""
## pypiwrap

A Python API wrapper for the Python Package Index.
"""

from .client import SimpleClient, PyPIClient

__name__ = "pypiwrap"
__author__ = 'Angel Carias'
__license__ = 'MIT'
__version__ = '1.1.0'
